require 'spec_helper'
require 'blender'
require 'blender/discoveries/serf'
require 'blender/serf_dsl'

describe Blender::Discovery::Serf do
  let(:scheduler) do
    Blender::Scheduler.new('test').extend Blender::SerfDSL
  end
  let(:discovery){described_class.new(host: '1.2.3.4')}

  it '#search' do
    conn = double('connection')
    response = double('response', body: {'Members'=> [{'Name'=>'a'}]})
    expect(conn).to receive(:members_filtered).with({}, "alive", "foo").and_return(response)
    expect(Serfx).to receive(:connect).with(host: '1.2.3.4').and_yield(conn)
    expect(discovery.search(name: 'foo')).to eq(['a'])
  end
    describe'#serf_task' do
      subject(:task){scheduler.tasks.first}
      before do
        scheduler.serf_task('test') do
          members ['b']
          query 'foo'
          payload 'bar'
          no_ack true
        end
      end
      it 'should have correct host list' do
        expect(task.hosts).to eq(['b'])
      end
      it 'should use the serf task subclass' do
        expect(task).to be_kind_of(Blender::Task::Serf)
      end
      it 'should use the serquery inner class for command' do
        expect(task.command).to be_kind_of(Blender::Task::Serf::SerfQuery)
      end
      it 'should use serf driver subclass' do
        expect(task.driver).to be_kind_of(Blender::Driver::Serf)
      end
      it 'should allow setting up serf query and payload' do
        expect(task.command.query).to eq('foo')
        expect(task.command.payload).to eq('bar')
      end
    end
end
