require 'spec_helper'
require 'blender/handlers/base'
require 'blender/cli'

describe Blender::Driver::Serf do
  let(:driver) do
    described_class.new(
      events: Blender::Handlers::Base.new,
      host: 'foo',
      port: 123,
      authkey: 'xyz'
    )
  end
  let(:tag_driver) do
    described_class.new(
      events: Blender::Handlers::Base.new,
      host: 'foo',
      port: 123,
      authkey: 'xyz',
      filter_by: :tag,
      filter_tag: 'fake-tag'
    )
  end
  let(:hosts) {['h1']}
  let(:tasks){ Array.new(3){|n| create_serf_task("t#{n}")}}
  it '#execute serf query' do
    conn = double('connection')
    conn_opts = { host: 'foo', port: 123, authkey: 'xyz'}
    query_opts = [
      'test-query',
      'test-payload',
      {:FilterNodes=>["h1"], :Timeout=>15000000000}
    ]
    expect(conn).to receive(:query).with(*query_opts).and_yield(Object.new).exactly(3).times
    expect(Serfx).to receive(:connect).with(conn_opts).and_yield(conn).exactly(3).times
    driver.execute(tasks, hosts)
  end
  it '#execute serf query based on tags' do
    conn = double('connection')
    conn_opts = { host: 'foo', port: 123, authkey: 'xyz'}
    query_opts = [
      'test-query',
      'test-payload',
      { FilterTags: {"fake-tag"=>"h1"}, :Timeout=>15000000000}
    ]
    expect(conn).to receive(:query).with(*query_opts).and_yield(Object.new).exactly(3).times
    expect(Serfx).to receive(:connect).with(conn_opts).and_yield(conn).exactly(3).times
    tag_driver.execute(tasks, hosts)
  end
  context '#CLI' do
    it '#sync' do
      conn = double('serf connection')
      ev = double('serf event')
      allow(conn).to receive(:query).with('test', 'start', Timeout: 7000000000, FilterNodes: ['node1']).and_yield(ev)
      allow(conn).to receive(:query).with('test', 'start', Timeout: 7000000000, FilterNodes: ['node2']).and_yield(ev)
      allow(conn).to receive(:query).with('test', 'start', Timeout: 7000000000, FilterNodes: ['node3']).and_yield(ev)
      allow(Serfx).to receive(:connect).with(authkey: 'foobar').and_yield(conn)
      Blender::CLI.start(%w{-f spec/data/example.rb})
    end
    it '#async' do
      conn = double('serf connection')
      ev = double('serf event')
      allow(ev).to receive(:body).and_return({'success' => true})
      %w{start reap check}.each do |payload|
        allow(conn).to receive(:query).with('chef', payload , Timeout: 5000000000, FilterNodes: ['node1']).and_yield(ev)
      end
      allow(Serfx).to receive(:connect).with(authkey: 'asyncexample').and_yield(conn)
      expect(Retriable).to receive(:retriable).with(max_elapsed_time: 300).and_yield
      Blender::CLI.start(%w{-f spec/data/async_example.rb})
    end
  end
end
